using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Tao.OpenGl;
using Tao.Platform.Windows;

namespace Sphere
{
    public partial class SphereForm : Form
    {
        private int width = 800;
        private int height = 600;
        private double thetaX = 0,thetaY=0,thetaZ=0;
        private Rectangle rec;
        private bool esc = false;
        private double x;
        private double y;
        private double z;
        private double aspRatio=800.0/600.0;
        private double xEye = 2;
        private double yEye = 8;
        private double zEye = 8;
        Sphere sphere = new Sphere();
        float[] lightPos ={ -2, 4, 3, 0.0f };
        //float[] ambientLight ={ 0.2f, 0.2f, 0.2f, 0.0f };
        //float[] diffuseLight ={ 0.972f, 0.12f, 0.05f, 0.0f };
        //float[] specularLight ={ 0.8f, 0.8f, 0.8f, 0.0f };
                    //Brass
        float[] ambientLight = { 0.329412f, 0.223529f, 0.027451f, 0.0f };
        float[] diffuseLight = { 0.980392f, 0.168627f, 0.013725f, 0.0f };
        float[] specularLight = { 0.992157f, 0.941176f, 0.807843f, 0.0f };


        public SphereForm()
        {
            InitializeComponent();
            glControl.InitializeContexts();
            initializeOpenGl();
        }
        public void initializeOpenGl()
        {
            this.SetBounds(0, 0, width, height);
            Gl.glViewport(0, 0, width, height);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            //Gl.glOrtho(-4 * aspRatio, 4 * aspRatio, -4, 4, -20, 20);
            Glu.gluPerspective(60, aspRatio, 1, 100);

            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(xEye, yEye, zEye, 0, 0, 0, 0, 1, 0);

            Gl.glClearColor(0, 0, 0, 1);
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glShadeModel(Gl.GL_SMOOTH);
            
            initLights();
        }
        public void initLights()
        {
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, lightPos);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambientLight);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diffuseLight);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, specularLight);
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);
        }
        private void Form1_Resize(object sender, EventArgs e)
        {
            rec = this.ClientRectangle;
            width = rec.Width;
            height = rec.Height;
            Gl.glViewport(0, 0, width, height);

            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glLoadIdentity();
            try
            {
                if (width > height)
                {
                    aspRatio = (double)width / height;
                   // Gl.glOrtho(-4 * aspRatio, 4 * aspRatio, -4, 4, -20, 20);
                }
                else
                {
                    aspRatio = (double)height / width;
                   // Gl.glOrtho(-4, 4, -4 * aspRatio, 4 * aspRatio, -20, 20);
                }
                Glu.gluPerspective(60, aspRatio, 1, 100);

            }
            catch (DivideByZeroException dbz)
            {
                dbz.ToString();
            }

            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(xEye, yEye, zEye, 0, 0, 0, 0, 1, 0);
        }
        private void glControl_Paint(object sender, PaintEventArgs e)
        {
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);          
            drawSurfaceMesh();           
            Gl.glPushMatrix();
                Gl.glRotated(thetaX, 1, 0, 0);
                Gl.glRotated(thetaY, 0, 1, 0);
                Gl.glRotated(thetaZ, 0, 0, 1);
                Gl.glTranslated(3.5, 0, 0);
                sphere.draw(1.8, 50);
            Gl.glPopMatrix();
            thetaX+=1;
            thetaY += 1.7;
            thetaZ += 2.3;
            if (thetaX < 720)
            {
                glControl.Invalidate();
            }
            else
            {
                thetaX = 0;
                thetaY = 0;
                thetaZ = 0;
                return;
            }
        }
        private void drawSurfaceMesh()
        {
            Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glColor3d(0.05, 1.0, 0.0);
            z = 30.0;
            for (x = -30; x < 30; x += 0.4)
            {
                Gl.glBegin(Gl.GL_LINES);
                    Gl.glVertex3d(x, 0.01, z);
                    Gl.glVertex3d(x, 0.01, -z);
                Gl.glEnd();
            }
            x = 30;
            for (z = -30; z < 30; z += 0.4)
            {
                Gl.glBegin(Gl.GL_LINES);
                    Gl.glVertex3d(x, 0.01, z);
                    Gl.glVertex3d(-x, 0.01, z);
                Gl.glEnd();
            }
            Gl.glEnable(Gl.GL_LIGHTING);
        }

        private void glControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.A)
            {
                yEye += 0.05;
                Gl.glMatrixMode(Gl.GL_MODELVIEW);
                Gl.glLoadIdentity();
                Glu.gluLookAt(xEye, yEye, zEye, 0, 0, 0, 0, 1, 0);
            }
            else if (e.KeyCode == Keys.Z)
            {
                yEye -= 0.05;
                Gl.glMatrixMode(Gl.GL_MODELVIEW);
                Gl.glLoadIdentity();
                Glu.gluLookAt(xEye, yEye, zEye, 0, 0, 0, 0, 1, 0);
            }
            glControl.Invalidate();
        }
    }
    class Sphere
    {
        public double[, ,] patchPoints;
        public double[, ,] normals;
        public int NUM = 20;

        public void selectPatchPoints(int num)
        {
            int m = 0, n = 0;
            NUM = num;
            double theta;
            double phi = 0.0;
            
            double diff = Math.PI / NUM;
            patchPoints = new double[NUM + 1, 2 * NUM + 2, 3];
            normals = new double[NUM + 1, 2 * NUM + 2, 3];

            //for (theta = diff; theta <= Math.PI; theta += diff, m++)
            for (phi = 0.0; phi <= 2 * Math.PI; phi += diff, n++)
            {
                //n = 0;
                m = 0;
                //for (phi = 0.0; phi <= 2 * Math.PI; phi += diff, n++)
                for (theta = diff; theta <= Math.PI; theta += diff, m++)
                {
                    patchPoints[m, n, 0] = Math.Sin(theta) * Math.Cos(phi);     //x
                    patchPoints[m, n, 1] = Math.Sin(theta) * Math.Sin(phi);     //y
                    patchPoints[m, n, 2] = Math.Cos(theta);                     //z
                    //norm = Math.Sqrt(patchPoints[m, n, 0] * patchPoints[m, n, 0] +
                    //                patchPoints[m, n, 1] * patchPoints[m, n, 1] +
                    //                patchPoints[m, n, 2] * patchPoints[m, n, 2]);
                    normals[m, n, 0] = patchPoints[m, n, 0];///norm;
                    normals[m, n, 1] = patchPoints[m, n, 1];///norm;
                    normals[m, n, 2] = patchPoints[m, n, 2];///norm;
                }
                patchPoints[m, n, 0] = Math.Sin(theta) * Math.Cos(phi);     //x
                patchPoints[m, n, 1] = Math.Sin(theta) * Math.Sin(phi);     //y
                patchPoints[m, n, 2] = Math.Cos(theta);                     //z
               
                normals[m, n, 0] = patchPoints[m, n, 0];///norm;
                normals[m, n, 1] = patchPoints[m, n, 1];///norm;
                normals[m, n, 2] = patchPoints[m, n, 2];///norm;
            }
        }
        public void draw(double radius, int num)
        {
            int m = 0, n = 0;
            NUM = num;
            int NUM1 = NUM + 1, NUM2 = 2 * NUM + 2;
            double[, ,] points = new double[NUM1, NUM2, 3];
            selectPatchPoints(num);
            for (m = 0; m <NUM1; m++)
            {
                for (n = 0; n <NUM2; n++)
                {
                    points[m, n, 0] = radius * patchPoints[m, n, 0];
                    points[m, n, 1] = radius * patchPoints[m, n, 1];
                    points[m, n, 2] = radius * patchPoints[m, n, 2];
                }
            }
            Gl.glBegin(Gl.GL_TRIANGLE_STRIP);//North Pole
            for (int k = 0; k < NUM2; k++)
            {
                //Gl.glNormal3d(0,0,radius);

                Gl.glNormal3d(normals[0, k, 0], normals[0, k, 1], normals[0, k, 2]);
                Gl.glVertex3d(points[0, k, 0], points[0, k, 1], points[0, k, 2]);
                Gl.glVertex3d(0, 0, radius);
            }
            Gl.glEnd();

            Gl.glBegin(Gl.GL_TRIANGLE_STRIP);//South Pole
            for (int k = 0; k < NUM2; k++)
            {
                Gl.glNormal3d(0,0,-radius);

                //Gl.glNormal3d(normals[0, k, 0], normals[0, k, 1], normals[0, k, 2]);
                Gl.glVertex3d(points[0, k, 0], points[0, k, 1], points[0, k, 2]);
                Gl.glVertex3d(0, 0, -radius);
            }
            Gl.glEnd();

            Gl.glBegin(Gl.GL_QUAD_STRIP);//Other than Poles
            for (m = 0; m < NUM; m++)
            {
                for (n = 0; n < NUM2; n++)
                {
                    Gl.glNormal3d(normals[m, n, 0], normals[m, n, 1], normals[m, n, 2]);
                    Gl.glVertex3d(points[m, n, 0], points[m, n, 1], points[m, n, 2]);
                    Gl.glNormal3d(normals[m + 1, n, 0], normals[m + 1, n, 1], normals[m + 1, n, 2]);
                    Gl.glVertex3d(points[m + 1, n, 0], points[m + 1, n, 1], points[m + 1, n, 2]);
                }
            }
            Gl.glEnd();
        }
    }
}